#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['tuxbake']

package_data = \
{'': ['*']}

install_requires = \
['tuxmake >=1.2.0']

entry_points = \
{'console_scripts': ['tuxbake = tuxbake.__main__:main']}

setup(name='tuxbake',
      version='0.0.8',
      description='Command line tool to build OpenEmbedded and Yocto images',
      author='Vishal Bhoj',
      author_email='vishal.bhoj@linaro.org',
      url='https://tuxbake.org/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
